<?php
/* Copyright (C) 2010      Regis Houssin        <regis.houssin@inodbox.com>
 * Copyright (C) 2015      Frederic France      <frederic.france@free.fr>
 * Copyright (C) 2016-2017 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2024		MDW							<mdeweerd@users.noreply.github.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

/**
 * \file       htdocs/suppliercontract/core/boxes/box_suppliercontracts.php
 * \ingroup    suppliercontract
 * \brief      Module de generation de l'affichage de la box suppliercontracts
 */

include_once DOL_DOCUMENT_ROOT . '/core/boxes/modules_boxes.php';


/**
 * Class to manage the box to show last contracts
 */
class box_suppliercontracts extends ModeleBoxes
{
	public $boxcode = "lastsuppliercontracts";
	public $boximg = "object_contract";
	public $boxlabel = "BoxLastSupplierContracts";
	public $depends = array("suppliercontract"); // conf->suppliercontract->enabled

	/**
	 *  Constructor
	 *
	 * @param DoliDB $db Database handler
	 * @param string $param More parameters
	 */
	public function __construct($db, $param)
	{
		global $user;

		$this->db = $db;

		$this->hidden = !($user->hasRight('suppliercontract', 'lire'));
	}

	/**
	 *  Load data for box to show them later
	 *
	 * @param int $max Maximum number of records to load
	 * @return    void
	 */
	public function loadBox($max = 5)
	{
		global $user, $langs, $conf;

		$this->max = $max;

		//include_once DOL_DOCUMENT_ROOT . '/suppliercontract/class/suppliercontract.class.php';
		dol_include_once('/suppliercontract/class/suppliercontract.class.php');

		$this->info_box_head = array(
			'text' => '<span class="valignmiddle">' . $langs->trans("BoxTitleLastSupplierContracts", $max) . '</span><a class="paddingleft valignmiddle" href="' . dol_buildpath('/suppliercontract/list.php', 1) . '?sortfield=c.tms&sortorder=DESC"><span class="badge">...</span></a>'
		);

		if ($user->hasRight('suppliercontract', 'lire')) {
			$suppliercontractstatic = new SupplierContract($this->db);
			$thirdpartytmp = new Societe($this->db);

			$sql = "SELECT s.nom as name, s.rowid as socid, s.email, s.client, s.fournisseur, s.code_client, s.code_fournisseur, s.code_compta, s.code_compta_fournisseur,";
			$sql .= " c.rowid, c.ref, c.status as status, c.date_contract, c.date_creation, c.tms as date_modification,";
			$sql .= " c.ref_customer, c.ref_supplier";
			$sql .= " FROM " . MAIN_DB_PREFIX . "societe as s, " . MAIN_DB_PREFIX . "suppliercontract as c";
			if (!$user->hasRight('societe', 'client', 'voir')) {
				$sql .= ", " . MAIN_DB_PREFIX . "societe_commerciaux as sc";
			}
			$sql .= " WHERE c.fk_soc = s.rowid";
			$sql .= " AND c.entity = " . $conf->entity;
			if (!$user->hasRight('societe', 'client', 'voir')) {
				$sql .= " AND s.rowid = sc.fk_soc AND sc.fk_user = " . ((int)$user->id);
			}
			if ($user->socid) {
				$sql .= " AND s.rowid = " . ((int)$user->socid);
			}
			if (getDolGlobalString('MAIN_LASTBOX_ON_OBJECT_DATE')) {
				$sql .= " ORDER BY c.date_contract DESC, c.ref DESC ";
			} else {
				$sql .= " ORDER BY c.tms DESC, c.ref DESC ";
			}
			$sql .= $this->db->plimit($max, 0);

			$resql = $this->db->query($sql);
			if ($resql) {
				$num = $this->db->num_rows($resql);
				$now = dol_now();

				$line = 0;

				$langs->loadLangs(array("contracts", "suppliercontracts@suppliercontract"));

				while ($line < $num) {
					$objp = $this->db->fetch_object($resql);

					$datec = $this->db->jdate($objp->date_creation);
					$datem = $this->db->jdate($objp->date_modification);
					$late = '';

					$suppliercontractstatic->status = $objp->status;
					$suppliercontractstatic->id = $objp->rowid;
					$suppliercontractstatic->ref = $objp->ref;
					$suppliercontractstatic->ref_customer = $objp->ref_customer;
					$suppliercontractstatic->ref_supplier = $objp->ref_supplier;
					$result = $suppliercontractstatic->fetch_lines();

					$thirdpartytmp->name = $objp->name;
					$thirdpartytmp->id = $objp->socid;
					$thirdpartytmp->email = $objp->email;
					$thirdpartytmp->client = $objp->client;
					$thirdpartytmp->fournisseur = $objp->fournisseur;
					$thirdpartytmp->code_client = $objp->code_client;
					$thirdpartytmp->code_fournisseur = $objp->code_fournisseur;
					$thirdpartytmp->code_compta = $objp->code_compta;
					$thirdpartytmp->code_compta_client = $objp->code_compta;
					$thirdpartytmp->code_compta_fournisseur = $objp->code_compta_fournisseur;

					// fin_validite is no more on supplier contract but on services
					// if ($objp->status == 1 && $dateterm < ($now - $conf->suppliercontract->cloture->warning_delay)) { $late = img_warning($langs->trans("Late")); }

					$this->info_box_contents[$line][] = array(
						'td' => 'class="nowraponall"',
						'text' => $suppliercontractstatic->getNomUrl(1),
						'text2' => $late,
						'asis' => 1
					);

					$this->info_box_contents[$line][] = array(
						'td' => 'class="tdoverflowmax150 maxwidth150onsmartphone"',
						'text' => $thirdpartytmp->getNomUrl(1),
						'asis' => 1
					);

					$this->info_box_contents[$line][] = array(
						'td' => 'class="center nowraponall" title="' . dol_escape_htmltag($langs->trans("DateModification") . ': ' . dol_print_date($datem, 'dayhour', 'tzuserrel')) . '"',
						'text' => dol_print_date($datem, 'day', 'tzuserrel'),
					);

					$this->info_box_contents[$line][] = array(
						'td' => 'class="nowraponall right"',
						'text' => $suppliercontractstatic->getLibStatut(7),
						'asis' => 1,
					);

					$line++;
				}

				if ($num == 0) {
					$this->info_box_contents[$line][0] = array(
						'td' => 'class="center"',
						'text' => '<span class="opacitymedium">' . $langs->trans("NoRecordedSupplierContracts") . '</span>'
					);
				}

				$this->db->free($resql);
			} else {
				$this->info_box_contents[0][0] = array(
					'td' => '',
					'maxlength' => 500,
					'text' => ($this->db->error() . ' sql=' . $sql),
				);
			}
		} else {
			$this->info_box_contents[0][0] = array(
				'td' => 'class="nohover left"',
				'text' => '<span class="opacitymedium">' . $langs->trans("ReadPermissionNotAllowed") . '</span>'
			);
		}
	}


	/**
	 *    Method to show box.  Called when the box needs to be displayed.
	 *
	 * @param    ?array<array{text?:string,sublink?:string,subtext?:string,subpicto?:?string,picto?:string,nbcol?:int,limit?:int,subclass?:string,graph?:int<0,1>,target?:string}>   $head       Array with properties of box title
	 * @param    ?array<array{tr?:string,td?:string,target?:string,text?:string,text2?:string,textnoformat?:string,tooltip?:string,logo?:string,url?:string,maxlength?:int,asis?:int<0,1>}> $contents Array with properties of box lines
	 * @param int<0,1> $nooutput No print, only return string
	 * @return    string
	 */
	public function showBox($head = null, $contents = null, $nooutput = 0)
	{
		return parent::showBox($this->info_box_head, $this->info_box_contents, $nooutput);
	}
}
